<?php defined('_JEXEC') or die('Restricted access');

require_once (JPATH_ADMINISTRATOR.'/components/com_j2store/library/plugins/payment.php');
//require_once (JPATH_SITE.'/components/com_j2store/helpers/utilities.php');

class plgJ2StorePayment_parspal extends J2StorePaymentPlugin
{
	/**
	 * @var $_element  string  Should always correspond with the plugin's filename,
	 *                         forcing it to be unique
	 */
    var $_element    = 'payment_parspal';

	/**
	 * Constructor
	 *
	 * For php4 compatability we must not use the __constructor as a constructor for plugins
	 * because func_get_args ( void ) returns a copy of all passed arguments NOT references.
	 * This causes problems with cross-referencing necessary for the observer design pattern.
	 *
	 * @param object $subject The object to observe
	 * @param 	array  $config  An array that holds the plugin configuration
	 * @since 1.5
	 */
	function plgJ2StorePayment_parspal(& $subject, $config) {
		parent::__construct($subject, $config);
		$this->loadLanguage( '', JPATH_ADMINISTRATOR );
	}


    /**
     * @param $data     array       form post data
     * @return string   HTML to display
     */
    function _prePayment( $data )
    {
		// get component params
		$params = JComponentHelper::getParams('com_j2store');

        // prepare the payment form

        $vars = new JObject();
        $vars->order_id = $data['order_id'];
        $vars->orderpayment_id = $data['orderpayment_id'];
        $vars->orderpayment_amount = $data['orderpayment_amount'];

        $vars->orderpayment_type = $this->_element;

        //get the order
        $order = JTable::getInstance('Orders', 'Table');
        $order->load($data['orderpayment_id']);

        $items = $order->getItems();

        //get the j2store version
        $xmlfile = JPATH_ADMINISTRATOR.'/components/com_j2store/manifest.xml';
        $xml = JFactory::getXML($xmlfile);
        $version=(string)$xml->version;

        foreach ($items as $item)
        {
        	$desc = $item->orderitem_name;

        	//sku
        	if (!empty($item->orderitem_sku))
        	{
        		$desc .= ' | '.JText::_('J2STORE_SKU').': '.$item->orderitem_sku;
        	}

        	//productoptions
        	if (!empty($item->orderitem_attribute_names)) {
        		//first convert from JSON to array
        		if( version_compare( $version, '2.5.0', 'lt' ) ) {
        			$desc .=' | '.$item->orderitem_attribute_names;
        		} else {
	        		$registry = new JRegistry;
	        		$registry->loadString($item->orderitem_attribute_names, 'JSON');
	        		$product_options = $registry->toObject();
	        		if(count($product_options) >0 ) {
	        			foreach ($product_options as $option) {
	        				$desc .=' | '.$option->name.':'.$option->value;
	        			}
        			}
        		}
        	}

        	$item->_description = $desc;
        }

        $vars->order = $order;
        $vars->orderitems = $items;

        // set payment plugin variables
        $vars->apiKey = trim($this->_getParam('apiKey'));
        $vars->post_url = $this->_getPostUrl();
        $vars->return_url = JURI::root()."index.php?option=com_j2store&view=checkout&task=confirmPayment&orderpayment_type=".$this->_element."&paction=display_message";
        $vars->cancel_url = JURI::root()."index.php?option=com_j2store&view=checkout&task=confirmPayment&orderpayment_type=".$this->_element."&paction=cancel";
        $vars->notify_url = JURI::root()."index.php?option=com_j2store&view=checkout&task=confirmPayment&orderpayment_type=".$this->_element."&paction=process&tmpl=component";
        //$vars->currency_code = $this->_getParam( 'currency', 'USD' );
        $vars->currency_code = $params->get('currency_code');

        // set variables for user info
        $vars->first_name   = $data['orderinfo']['billing_first_name'];
        $vars->last_name    = $data['orderinfo']['billing_last_name'];
        $vars->email        = $data['orderinfo']['user_email'];
        $vars->address_1    = $data['orderinfo']['billing_address_1'];
        $vars->address_2    = $data['orderinfo']['billing_address_2'];
        $vars->city         = $data['orderinfo']['billing_city'];
        $vars->country      = $data['orderinfo']['billing_country_name'];
        $vars->region       = $data['orderinfo']['billing_zone_name'];
        $vars->postal_code  = $data['orderinfo']['billing_zip'];

        $html = $this->_getLayout('prepayment', $vars);
        return $html;
    }

    /**
     * Processes the payment form
     * and returns HTML to be displayed to the user
     * generally with a success/failed message
     *
     * @param $data     array       form post data
     * @return string   HTML to display
     */
    function _postPayment( $data )
    {
        // Process the payment
        $paction = JRequest::getVar('paction');

        $vars = new JObject();

        switch ($paction)
        {
            case "display_message":
                $vars->message = JText::_('J2STORE_PARSPAL_MESSAGE_PAYMENT_ACCEPTED_FOR_VALIDATION');
                $html = $this->_getLayout('message', $vars);
                $html .= $this->_displayArticle();
              break;
            case "process":
                $vars->message = $this->_process();
                //$html = $this->_getLayout('message', $vars);
                //echo $html; // TODO Remove this
                //$app = JFactory::getApplication();
                //$app->close();
				if ($vars->message=='')
				{
					header('location: '. JURI::root()."index.php?option=com_j2store&view=checkout&task=confirmPayment&orderpayment_type=".$_REQUEST['orderpayment_type']."&paction=display_message");
					exit;die;
				}
				else
				{
					header('location: '. JURI::root()."index.php?option=com_j2store&view=checkout&task=confirmPayment&orderpayment_type=".$_REQUEST['orderpayment_type']."&paction=cancel");
					exit;die;
				}

              break;
            case "cancel":
                $vars->message = JText::_( 'J2STORE_PARSPAL_MESSAGE_CANCEL' );
                $html = $this->_getLayout('message', $vars);
              break;
            default:
                $vars->message = JText::_( 'J2STORE_PARSPAL_MESSAGE_INVALID_ACTION' );
                $html = $this->_getLayout('message', $vars);
              break;
        }

        return $html;
    }

    /**
     * Prepares variables for the payment form
     *
     * @return unknown_type
     */
    function _renderForm( $data )
    {
        $user = JFactory::getUser();
        $vars = new JObject();

        $html = $this->_getLayout('form', $vars);

        return $html;
    }

    /************************************
     * Note to 3pd:
     *
     * The methods between here
     * and the next comment block are
     * specific to this payment plugin
     *
     ************************************/

    /**
     * Gets the ParsPal gateway URL
     *
     * @param boolean $full
     * @return string
     * @access protected
     */
    function _getPostUrl($full = true)
    {
		return 'https://www.parspal.com';
    }


    /**
     * Gets the value for the ParsPal variable
     *
     * @param string $name
     * @return string
     * @access protected
     */
    function _getParam( $name, $default='' )
    {
    	$return = $this->params->get($name, $default);

    	$sandbox_param = "sandbox_$name";
    	$sb_value = $this->params->get($sandbox_param);
        if ($this->params->get('sandbox') && !empty($sb_value))
        {
            $return = $this->params->get($sandbox_param, $default);
        }

        return $return;
    }

    /**
     * Validates the IPN data
     *
     * @param array $data
     * @return string Empty string if data is valid and an error message otherwise
     * @access protected
     */
    function _validateIPN( $amount )
    {
		require_once(dirname(__FILE__) . '/parspal_payment_helper.class.php');
		$p = new ParsPal_Payment_Helper($this->params->get('apiKey'));
		$r = $p->paymentVerify($_POST, $amount);
		if ($r)
		{
			$this->txn_id = $p->txn_id;
			return '';
		}
		else
		{
			return $p->error . ' . ' . JText::_('J2STORE_PARSPAL_ERROR_IPN_VALIDATION');
		}
    }

	/**
	 *
	 * @return HTML
	 */
	function _process()
	{
		$data = JRequest::get('post');

      JTable::addIncludePath( JPATH_ADMINISTRATOR.'/components/com_j2store/tables' );
      $orderpayment = JTable::getInstance('Orders', 'Table');
      //remove this once you are live.
      //$oid = substr($data['reserve_id'], 2, 3);
      $orderpayment->load( $data['reserve_id'] );

		// validate the IPN info
		$error = $this->_validateIPN(intval(ceil($orderpayment->get('orderpayment_amount'))));
		if ($error == '')
		{
			$data['payment_status'] = 'Completed';
			$data['txn_id'] = $this->txn_id;
		}
		else
		{
			// ipn Validation failed
			$data['ipn_validation_results'] = $error;
			$data['payment_status'] = 'Failed';
		}

        // prepare some data
        $order_id           = (int) $data['order_id'];
        $orderpayment_id    = (int) $data['reserve_id'];
        $data['transaction_details'] = $this->_getFormattedTransactionDetails( $data );

			// process the payment based on its type
			$payment_error = '';
			// Payment received for multiple items; source is Express Checkout or the ParsPal Shopping Cart.
			$payment_error = $this->_processSale( $data, $error );

			if ($payment_error)
			{
				 // it seems like an error has occurred during the payment process
				 $error .= $error ? "\n" . $payment_error : $payment_error;
			}

        if ($error) {
            // send an emails to site's administrators with error messages
            $this->_sendErrorEmails($error, $data['transaction_details']);
            return $error;
        }


        // if here, all went well
        //$error = 'processed';
		return $error;
	}

    /**
     * Processes the sale payment
     *
     * @param array $data IPN data
     * @return boolean Did the IPN Validate?
     * @access protected
     */
    function _processSale( $data, $ipnValidationFailed='' )
    {
        /*
         * validate the payment data
         */
        $errors = array();

        if (!empty($ipnValidationFailed))
        {
        	$errors[] = $ipnValidationFailed;
        }

        if (empty($data['reserve_id']))
        {
            $errors[] = JText::_('J2STORE_PARSPAL_MESSAGE_INVALID_ORDERPAYMENTID');
            return count($errors) ? implode("\n", $errors) : '';
        }

        // load the orderpayment record and set some values
        JTable::addIncludePath( JPATH_ADMINISTRATOR.'/components/com_j2store/tables' );
        $orderpayment = JTable::getInstance('Orders', 'Table');

        //remove this once you are live.
        //$oid = substr($data['reserve_id'], 2, 3);

        $orderpayment->load( $data['reserve_id'] );
        $orderpayment->transaction_details  = $data['transaction_details'];
        $orderpayment->transaction_id       = $data['txn_id'];
        $orderpayment->transaction_status   = $data['payment_status'];

        // check the stored amount against the payment amount
        //$stored_amount = J2StoreUtilities::number( $orderpayment->get('orderpayment_amount'), array( 'thousands'=>'', 'num_decimals'=>'2', 'decimal'=>'.' ) );

        //set a default status to it
        $orderpayment->order_state = JText::_('J2STORE_PENDING'); // PENDING
        $orderpayment->order_state_id = 4; // PENDING

        // set the order's new status
        if (count($errors))
        {
            // if an error occurred
            $orderpayment->order_state = JText::_('J2STORE_FAILED'); // FAILED
            $orderpayment->order_state_id = 3; //FAILED
        }
        elseif (@$data['payment_status'] == 'Pending')
        {
            // if the transaction has the "pending" status,
            $orderpayment->order_state = JText::_('J2STORE_PENDING'); // PENDING
            $orderpayment->order_state_id = 4; //PENDING

        }
        else
        {
            $orderpayment->order_state = trim(JText::_('J2STORE_CONFIRMED')); // CONFIRMED
            $orderpayment->order_state_id = 1; //CONFIRMED
            $orderpayment->paypal_status = @$data['payment_status']; // Paypal's original status
            JLoader::register( 'J2StoreHelperCart', JPATH_SITE.'/components/com_j2store/helpers/cart.php');
            // remove items from cart
            J2StoreHelperCart::removeOrderItems( $orderpayment->id );

        }

        // save the orderpayment
        if($orderpayment->save())
        {

		} else {
        	$errors[] = $orderpayment->getError();
        }

        // let us inform the user that the payment is successful
        require_once (JPATH_SITE.'/components/com_j2store/helpers/orders.php');
        J2StoreOrdersHelper::sendUserEmail($orderpayment->user_id, $orderpayment->order_id, $data['payment_status'], $orderpayment->order_state, $orderpayment->order_state_id);

        return count($errors) ? implode("\n", $errors) : '';
    }

    /**
     * Formatts the payment data for storing
     *
     * @param array $data
     * @return string
     */
    function _getFormattedTransactionDetails( $data )
    {
        $separator = "\n";
        $formatted = array();

        foreach ($data as $key => $value)
        {
            if ($key != 'view' && $key != 'layout')
            {
                $formatted[] = $key . ' = ' . $value;
            }
        }

        return count($formatted) ? implode("\n", $formatted) : '';
    }

    /**
     * Sends error messages to site administrators
     *
     * @param string $message
     * @param string $paymentData
     * @return boolean
     * @access protected
     */
    function _sendErrorEmails($message, $paymentData)
    {
        $mainframe = JFactory::getApplication();

        // grab config settings for sender name and email
        $config     = JComponentHelper::getParams('com_j2store');
        $mailfrom   = $config->get( 'emails_defaultemail', $mainframe->getCfg('mailfrom') );
        $fromname   = $config->get( 'emails_defaultname', $mainframe->getCfg('fromname') );
        $sitename   = $config->get( 'sitename', $mainframe->getCfg('sitename') );
        $siteurl    = $config->get( 'siteurl', JURI::root() );

        $recipients = $this->_getAdmins();
        $mailer = JFactory::getMailer();

        $subject = JText::sprintf('J2STORE_PARSPAL_EMAIL_PAYMENT_NOT_VALIDATED_SUBJECT', $sitename);

        foreach ($recipients as $recipient)
        {
            $mailer = JFactory::getMailer();
            $mailer->addRecipient( $recipient->email );

            $mailer->setSubject( $subject );
            $mailer->setBody( JText::sprintf('J2STORE_PARSPAL_EMAIL_PAYMENT_FAILED_BODY', $recipient->name, $sitename, $siteurl, $message, $paymentData) );
            $mailer->setSender(array( $mailfrom, $fromname ));
            $sent = $mailer->send();
        }

        return true;
    }

    /**
     * Gets admins data
     *
     * @return array|boolean
     * @access protected
     */
    function _getAdmins()
    {
        $db = JFactory::getDBO();
        $query = $db->getQuery(true);
        $query->select('u.name,u.email');
        $query->from('#__users AS u');
        $query->join('LEFT', '#__user_usergroup_map AS ug ON u.id=ug.user_id');
        $query->where('u.sendEmail = 1');
        $query->where('ug.group_id = 8');

        $db->setQuery($query);
        $admins = $db->loadObjectList();
        if ($error = $db->getErrorMsg()) {
            JError::raiseError(500, $error);
            return false;
        }

        return $admins;
    }


}

      /* TYPICAL RESPONSE FROM PAYPAL INCLUDES:
       * mc_gross=49.99
       * &protection_eligibility=Eligible
       * &address_status=confirmed
       * &payer_id=Q5HTJ93G8FQKC
       * &tax=0.00
       * &address_street=10101+Some+Street
       * &payment_date=12%3A13%3A19+Dec+05%2C+2008+PST
       * &payment_status=Completed
       * &charset=windows-1252
       * &address_zip=11259
       * &first_name=John
       * &mc_fee=1.75
       * &address_country_code=US
       * &address_name=John+Doe
       * &custom=some+custom+value
       * &payer_status=verified
       * &business=receiver%40domain.com
       * &address_country=United+States
       * &address_city=Some+City
       * &quantity=1
       * &payer_email=sender%40emaildomain.com
       * &txn_id=3JK16594EX581780W
       * &payment_type=instant
       * &payer_business_name=John+Doe
       * &last_name=Doe
       * &address_state=CA
       * &receiver_email=receiver%40domain.com
       * &payment_fee=1.75
       * &receiver_id=YG9UDRP6DE45G
       * &txn_type=web_accept
       * &item_name=Name+of+item
       * &mc_currency=USD
       * &item_number=Number+of+Item
       * &residence_country=US
       * &handling_amount=0.00
       * &transaction_subject=Subject+of+Transaction
       * &payment_gross=49.99
       * &shipping=0.00
       * &=
      */

    /**
     * VALID PAYMENT_STATUS VALUES returned from Paypal
     *
     * Canceled_Reversal: A reversal has been canceled. For example, you won a dispute with the customer, and the funds for the transaction that was reversed have been returned to you.
     * Completed: The payment has been completed, and the funds have been added successfully to your account balance.
     * Created: A German ELV payment is made using Express Checkout.
     * Denied: You denied the payment. This happens only if the payment was previously pending because of possible reasons described for the pending_reason variable or the Fraud_Management_Filters_x variable.
     * Expired: This authorization has expired and cannot be captured.
     * Failed: The payment has failed. This happens only if the payment was made from your customer�s bank account.
     * Pending: The payment is pending. See pending_reason for more information.
     * Refunded: You refunded the payment.
     * Reversed: A payment was reversed due to a chargeback or other type of reversal. The funds have been removed from your account balance and returned to the buyer. The reason for the reversal is specified in the ReasonCode element.
     * Processed: A payment has been accepted.
     * Voided: This authorization has been voided.
    */
